"addreg"<-
function(covar, trisk, status, variablenr = -1, labelofvariable = "", maintitle
	 = "", startattime = -1, stopattime = -1)
{
#Public domain Splus function for analyses of surrviel analyses data.
#Written by: 	      Harald Fekjaer, E-mail: hfe@math.uio.no
#For help se:         addreg.h
#See also:	      http://www.med.uio.no/imb/stat/addreg/		
#
#-----------Beginning of routine: -----------------
#Finding estimates:
	res <- addreg.find(covar, trisk, status, stopattime)	#
# Plotting:
	addreg.show(res, variablenr, labelofvariable, maintitle, startattime, 
		stopattime)
}
"addreg.est"<-
function(linaerestimate, variablenr = c(-1, -1), times, digitsest = 2, 
	digitsvar = 4)
{
#Public domain Splus function for analyses of surrviel analyses data.
#Written by: 	      Harald Fekjaer, E-mail: hfe@math.uio.no
#For help se:         addreg.est.h
#See also:	      http://www.med.uio.no/imb/stat/addreg/	
#
#-----------Beginning of routine: -----------------
	if(variablenr[1] == -1) {
		variablenr <- 0:(dim(linaerestimate)[2] - 2)
	}
	j <- 1
	for(i in 1:length(times)) {
		while(times[i] > linaerestimate[1, 1, j]) {
			j <- j + 1
		}
		cat("\nEstimat for time", times[i], "is:\n")
		for(nr in 1:length(variablenr)) {
			cat("Variable nr", variablenr[nr], ":", round(
				linaerestimate[1, variablenr[nr] + 2, j - 1], 
				digits = digitsest), "with varianse", round(
				linaerestimate[2, variablenr[nr] + 2, j - 1], 
				digits = digitsvar))
			cat("\n")
		}
	}
	cat("\n\n")
}
"addreg.find"<-
function(covar, trisk, status, stoptime = -1, stopcond = 0.001, stand = T, 
	na.rm = F)
{
#Public domain Splus function for analyses of surrviel analyses data.
#Written by: 	      Harald Fekjaer, E-mail: hfe@math.uio.no
#For help se:         addreg.find.h
#See also:	      http://www.med.uio.no/imb/stat/addreg/	
#
#-----------Beginning of routine: -----------------
#
#Setting local variables:
#
	if(is.vector(covar)) dimcovar <- c(length(covar), 1) else dimcovar <- 
			dim(covar)
	if(((dimcovar[1] == length(trisk)) && (length(trisk) == length(status))
		) == F) {
		cat("Error:\tVariables \"covar\", \"trisk\" and \"status\"\n")
		cat("        must have the same number of observations\n\n")
		return(0)
	}
# Testing for unknown values:	
	if((prod(is.na(covar) == F) == 0) || (prod(is.na(trisk) == F) == 0) || (
		prod(is.na(status) == F) == 0)) {
		if(na.rm) {
			notmissing <- c(rep(1, times = dimcovar[1]))
			i <- 1
			for(i in 1:dimcovar[1]) {
				notmissing[i] <- prod(is.na(covar[i,  ]) == F)
			}
			notmissing <- notmissing * (is.na(trisk) == F) * (is.na(
				status) == F)
			cat(dimcovar[1] - sum(notmissing), 
				" observations ignored because of missing values\n"
				)
			dimcovar[1] <- sum(notmissing)
			covar <- covar[notmissing == 1,  ]
			trisk <- trisk[notmissing == 1]
			status <- status[notmissing == 1]
		}
		else {
			cat("Error:\tMissing not allowed unless na.rm=T\n\n")
			return(0)
		}
	}
#		
#Sorting times and variables by time:
	if(is.vector(covar))
		covar <- covar[order(trisk)]
	else covar <- covar[order(trisk), 1:dimcovar[2]]
	status <- status[order(trisk)]
	trisk <- sort(trisk)
	if(stand) {
           if(is.matrix(covar))
                covar <- covar - t(matrix(rep(apply(covar, 2, mean), times= dimcovar[1]), dimcovar[2]))   
                else covar <-covar-mean(covar) 
        } 	
#Finding estimator:
#
	times <- sort(trisk)
	if(stoptime > -1) {
		times <- times[1:sum((times < stoptime) == T)]
	}
	lencom <- length(times)
	nuller <- rep(0, times = (dimcovar[2] + 1))
	estres <- nuller
	estresvar <- nuller
	res <- c(rep(NA, times = ((dimcovar[2] + 2) * lencom * 2)))
	dim(res) <- c(2, dimcovar[2] + 2, lencom)
	res[1, 1,  ] <- times
	res[2, 1,  ] <- times
	id <- matrix(c(rep(0, times = (dimcovar[1] * dimcovar[1]))), dimcovar[1
		],  )
	ymat <- matrix(c(rep(1, times = dimcovar[1]), covar), dimcovar[1],  )
	tyymat <- t(ymat) %*% ymat
	absdet <- abs(prod(svd(tyymat, nu = 0, nv = 0)$d))
	if(absdet < stopcond) {
		cat("Error: Matrix has too low rank\n")
		return(NA)
	}
	if(status[1] == 1) {
		xmat <- solve(tyymat) %*% t(ymat)
		estres <- estres + xmat[, 1]
		imat <- id
		imat[1, 1] <- 1
		estresvar <- estresvar + diag((xmat %*% imat %*% t(xmat)))
	}
	res[1, 2:(dimcovar[2] + 2), 1] <- estres
	res[2, 2:(dimcovar[2] + 2), 1] <- estresvar
	for(j in 2:lencom) {
		ymat[j - 1,  ] <- rep(0, times = (dimcovar[2] + 1))
		tyymat <- t(ymat) %*% ymat
		absdet <- abs(prod(svd(tyymat, nu = 0, nv = 0)$d))
		if(absdet < stopcond) {
			cat("Remark: Stopped at time ", times[j - 1], 
				" because of too low rank\n")
			res <- res[,  , 1:(j - 1)]
			return(res)
		}
		if(status[j] == 1) {
			xmat <- solve(tyymat) %*% t(ymat)
			estres <- estres + xmat[, j]
			imat <- id
			imat[j, j] <- 1
			estresvar <- estresvar + diag((xmat %*% imat %*% t(xmat
				)))
		}
		res[1, 2:(dimcovar[2] + 2), j] <- estres
		res[2, 2:(dimcovar[2] + 2), j] <- estresvar
	}
	res
}
"addreg.show"<-
function(linaerestimate, variablenr = c(-1, -1), labelofvariable = "", 
	maintitle = "", startattime = -1, stopattime = -1)
{
#Public domain Splus function for analyses of surrviel analyses data.
#Written by: 	      Harald Fekjaer, E-mail: hfe@math.uio.no
#For help se:         addreg.h
#See also:	      http://www.med.uio.no/imb/stat/addreg/		
#
#-----------Beginning of routine: -----------------
# Spliting in different plots:
	if(startattime != -1) {
		i <- 1
		if(linaerestimate[1, 1, dim(linaerestimate)[3]] < startattime)
			cat("Warning dataset don't include data for time", 
				starattime, "\n")
		else {
			while(linaerestimate[1, 1, i] < startattime) i <- i + 1
			linaerestimate <- linaerestimate[,  , i:dim(
				linaerestimate)[3]]
			firstrow <- linaerestimate[1, 2:dim(linaerestimate)[2], 
				1]
			for(j in 1:dim(linaerestimate)[3])
				linaerestimate[1, 2:dim(linaerestimate)[2], j] <- 
				  linaerestimate[1, 2:dim(linaerestimate)[2], j
				  ] - firstrow
			firstrow <- linaerestimate[2, 2:dim(linaerestimate)[2], 
				1]
			for(j in 1:dim(linaerestimate)[3])
				linaerestimate[2, 2:dim(linaerestimate)[2], j] <- 
				  linaerestimate[2, 2:dim(linaerestimate)[2], j
				  ] - firstrow
		}
	}
	if(stopattime != -1) {
		i <- 1
		if(linaerestimate[1, 1, dim(linaerestimate)[3]] < stopattime)
			cat("Warning dataset don't include data for time", 
				stopattime, "\n")
		else {
			while(linaerestimate[1, 1, i] < stopattime) i <- i + 1
			linaerestimate <- linaerestimate[,  , 1:(i - 1)]
		}
	}
	if(variablenr[1] == -1)
		variablenr <- (1:(dim(linaerestimate)[2] - 1)) - 1
	lenvar <- length(variablenr)
	if(lenvar > 1) {
		if(variablenr[1] == 0 && length(variablenr) > length(
			labelofvariable))
			labelofvariable <- c("Constant", labelofvariable)
		dimplot <- c(NA, NA)
		dimplot[1] <- ceiling(sqrt(lenvar))
		dimplot[2] <- ceiling(lenvar/dimplot[1])
		screens <- split.screen(figs = dimplot)
		for(i in 1:lenvar) {
			screen(screens[i])
			addreg.show(linaerestimate, variablenr[i], 
				labelofvariable[i], " ")
		}
		close.screen(, T)
		if(maintitle == "")
			maintitle <- "Additive regression of life times"
		title(maintitle)
	}
	else {
# Setting local parameters:
		dimest <- dim(linaerestimate)	#
		interval <- 2 * sqrt(linaerestimate[2, variablenr + 2,  ])
		maxintval <- max(linaerestimate[1, variablenr + 2,  ] + 
			interval)
		minintval <- min(linaerestimate[1, variablenr + 2,  ] - 
			interval)	# Plotting esimate:
		matplot(linaerestimate[1, 1,  ], linaerestimate[1, variablenr + 
			2,  ], type = "s", xlab = "Time", ylab = 
			labelofvariable, ylim = c(minintval, maxintval))	#
# Plotting confident intervalls:
		lines(linaerestimate[1, 1,  ], linaerestimate[1, variablenr + 2,
			] + interval, lty = 2)
		lines(linaerestimate[1, 1,  ], linaerestimate[1, variablenr + 2,
			] - interval, lty = 2)	#
# Setting title & vertical line in 0:
		abline(h = c(0, 0), lty = 4)
		if(maintitle == "")
			maintitle <- "Additive regression of life times"
		title(maintitle)
	}
}
"addreg.test"<-
function(covar, trisk, status, starttime = -1, stoptime = -1, stopcond = 0.001, 
	stand = T, na.rm = F)
{
#Public domain Splus function for analyses of surrviel analyses data.
#Written by: 	      Harald Fekjaer, E-mail: hfe@math.uio.no
#For help se:         addreg.find.h
#See also:	      http://www.med.uio.no/imb/stat/addreg/		
#
#-----------Beginning of routine: -----------------
#
#Setting local variables:
##
	if(is.vector(covar)) dimcovar <- c(length(covar), 1) else dimcovar <- 
			dim(covar)
	if(((dimcovar[1] == length(trisk)) && (length(trisk) == length(status))
		) == F) {
		cat("Error:\tVariables \"covar\", \"trisk\" and \"status\"\n")
		cat("        must have the same number of observations\n\n")
		return(0)
	}
	i <- 1	#
# Testing for unknown values:
#
	if((prod(is.na(covar) == F) == 0) || (prod(is.na(trisk) == F) == 0) || (
		prod(is.na(status) == F) == 0)) {
		if(na.rm) {
			notmissing <- c(rep(1, times = dimcovar[1]))
			for(i in 1:dimcovar[1]) {
				notmissing[i] <- prod(is.na(covar[i,  ]) == F)
			}
			notmissing <- notmissing * (is.na(trisk) == F) * (is.na(
				status) == F)
			cat(dimcovar[1] - sum(notmissing), 
				" observations ignored because of missing values\n"
				)
			dimcovar[1] <- sum(notmissing)
			covar <- covar[notmissing == 1,  ]
			trisk <- trisk[notmissing == 1]
			status <- status[notmissing == 1]
		}
		else {
			cat("Error:\tMissing not allowed unless na.rm=T\n\n")
			return(0)
		}
	}
#		
# Sorting times and variables by time:
#
	if(is.vector(covar))
		covar <- covar[order(trisk)]
	else covar <- covar[order(trisk), 1:dimcovar[2]]
	status <- status[order(trisk)]
	trisk <- sort(trisk)
	if(starttime > -1) {
		from <- sum((trisk < starttime)) + 1
		trisk <- trisk[from:dimcovar[1]]
		trisk <- trisk - starttime
		status <- status[from:dimcovar[1]]
		covar <- covar[from:dimcovar[1],  ]
		dimcovar[1] <- length(trisk)
	}
        if(stand) {
           if(is.matrix(covar))
                covar <- covar - t(matrix(rep(apply(covar, 2, mean), times = dimcovar[1]), dimcovar[2]))   
                else covar <-covar-mean(covar) 
        }  
	times <- sort(trisk)
	if(stoptime > -1) {
		times <- times[1:sum((times < stoptime) == T)]
	}
	lencom <- length(times)	#	
#Finding estimator:
#
	nuller <- rep(0, times = (dimcovar[2] + 1))
	estres <- nuller
	res <- nuller
	estresvar <- matrix(c(rep(0, times = ((dimcovar[2] + 1) * (dimcovar[2] + 
		1)))), dimcovar[2] + 1,  )
	kmatnull <- matrix(c(rep(0, times = ((dimcovar[2] + 1) * (dimcovar[2] + 
		1)))), dimcovar[2] + 1,  )
	ymat <- matrix(c(rep(1, times = dimcovar[1]), covar), dimcovar[1],  )
	imatnull <- matrix(c(rep(0, times = (dimcovar[1] * dimcovar[1]))), 
		dimcovar[1],  )
	tyymat <- t(ymat) %*% ymat
	absdet <- abs(prod(svd(tyymat, nu = 0, nv = 0)$d))
	if(absdet < stopcond) {
		cat("Error: Matrix has to low rank\n")
		return(0)
	}
	if(status[1] == 1) {
		xmat <- solve(tyymat) %*% t(ymat)
		kmat <- kmatnull
		diag(kmat) <- diag(solve(tyymat))
		kmat <- solve(kmat)
		ledd <- kmat %*% xmat
		estres <- estres + ledd[, 1]
		imat <- imatnull
		imat[1, 1] <- 1
		ledd <- kmat %*% xmat %*% imat %*% t(xmat) %*% kmat
		estresvar <- estresvar + ledd
	}
	for(i in 2:lencom) {
		ymat[i - 1,  ] <- rep(0, times = (dimcovar[2] + 1))
		tyymat <- t(ymat) %*% ymat
		eigenval <- eigen(tyymat)$values
		absdet <- abs(prod(svd(tyymat, nu = 0, nv = 0)$d))
		if(absdet < stopcond) {
			cat("Remark: Resultes are taken at time ", trisk[i - 1],
				"\n")
			res <- estres/sqrt(diag(estresvar))
			c1 <- c("Test-value:", round(res, digits = 4))
			absres <- abs(res)
			c2 <- c("P-value:   ", round((1 - pnorm(absres)) * 2, 
				digits = 4))
			cat(format(t(matrix(c(c1, c2), length(res) + 1)))[1,  ],
				"\n")
			cat(format(t(matrix(c(c1, c2), length(res) + 1)))[2,  ],
				"\n")
			return(0)
		}
		if(status[i] == 1) {
			xmat <- solve(tyymat) %*% t(ymat)
			kmat <- kmatnull
			diag(kmat) <- diag(solve(tyymat))
			kmat <- solve(kmat)
			ledd <- kmat %*% xmat
			estres <- estres + ledd[, i]
			imat <- imatnull
			imat[i, i] <- 1
			ledd <- kmat %*% xmat %*% imat %*% t(xmat) %*% kmat
			estresvar <- estresvar + ledd
		}
	}
	res <- estres/sqrt(diag(estresvar))
	c1 <- c("Test-value:", round(res, digits = 4))
	absres <- abs(res)
	c2 <- c("P-value:   ", round((1 - pnorm(absres)) * 2, digits = 4))
	cat(format(t(matrix(c(c1, c2), length(res) + 1)))[1,  ], "\n")
	cat(format(t(matrix(c(c1, c2), length(res) + 1)))[2,  ], "\n")
}
